#ifndef SESSIONDBUSREGISTER_H
#define SESSIONDBUSREGISTER_H

#include <QObject>
#include <QString>
#include <QStringList>
#include <QDebug>
#include <QDBusContext>
#include <QDBusConnection>
#include <QDBusConnectionInterface>
#include <QDBusReply>
#include "ukui-log4qt.h"
#include "CSingleton.h"


class SystemAdapter;

class SysDbusMng : public QObject
{
    Q_OBJECT
protected:
    explicit SysDbusMng(QObject *parent = nullptr);
    ~SysDbusMng();

public:
    int start(void);

    int stop(void);

    SystemAdapter *get_sys_adapter(){ return m_sys_adapter; }

public:
    bool registerClient(QMap<QString, QVariant>, QString);
    bool unregisterClient(QString);

    QString bluetoothKeyValue(unsigned int,QString);

signals:
    void adapterAddSignal(QMap<QString, QVariant>);
    void adapterAttrChanged(QString, QMap<QString, QVariant>);
    void adapterRemoveSignal(QString);
    void deviceAddSignal(QMap<QString, QVariant>);
    void deviceAttrChanged(QString, QMap<QString, QVariant>);
    void deviceRemoveSignal(QString, QMap<QString, QVariant>);
    void ActiveConnection(QString, QString, QString, int, int);
    void updateClient(void);
    void startPair(QMap<QString, QVariant>);
    void fileStatusChanged(QMap<QString, QVariant>);
    void fileReceiveSignal(QMap<QString, QVariant>);
    void clearBluetoothDev(QStringList);


    //音频控制信号
    void VolumeDown();
    void VolumeUp();
    void Next();
    void PlayPause();
    void Previous();
    void Stop();
    void Play();
    void Pause();
protected:
    void sendMultimediaControlButtonSignal(unsigned int);

private:
    SystemAdapter * m_sys_adapter = nullptr;

    QMap<unsigned int, long long> m_pressTimeMap;     //记录键值按下的时间
    QMap<unsigned int, long long> m_releaseTimeMap;   //记录键值释放的时间

    friend class SingleTon<SysDbusMng>;
};

typedef SingleTon<SysDbusMng>  SYSDBUSMNG;

#endif // SESSIONDBUSREGISTER_H
