#ifndef BLUETOOTHDBUS_H
#define BLUETOOTHDBUS_H

#include <glib.h>
#include <gio/gio.h>
#include <QString>
#include <QDebug>
#include <QStringList>
#include <QDBusConnection>
#include <QDBusReply>
#include <QDBusMessage>
#include <QMap>
#include <QProcess>
#include <QDBusConnectionInterface>
#include <QStandardPaths>
#include "config/config.h"

class BluetoothDbus : public QObject
{
    Q_OBJECT
public:
    explicit BluetoothDbus(QObject *parent = nullptr);

    ~BluetoothDbus();

    void setDeviceList(QStringList devicelist);

    void setDeviceType(QStringList typelist);

public:
    //DBus调用函数
    void InitDefaultAdapterDevicesAttr();

    bool getTrayIconShow();

    void setDefaultAdapterPower(bool power);

    void cancelFileTransfer(QString address, int opt);

    bool sendFiles(QString address, QStringList filelist);

    void openBluetoothSettings();

    void devConnect(const QString address);

    void devDisconnect(const QString address);

    void activeConnectionReply(QString dev, bool v);

    void replyFileReceiving(QString address, bool accept);

    void pairFuncReply(QString address, bool accept);

    bool isPowered();

    bool getExistAdapter();

    bool getDevPairStatus(QString address);

    QMap<QString, QMap<QString, QVariant>> getPairedDevices();
    QMap<QString, QMap<QString, QVariant>> getSendableDevices();

signals:
    void devRemoveSignal(QString);

    void devAttrChanged(QString, QMap<QString, QVariant>);

    void showTrayIcon(bool);

    void existAdapter(bool);

    void adapterAddSignal(QString);

    void adapterRemoveSignal(QString);

    void adapterChangedSignal();

    void adapterCanOperat(bool);

    void powerChangedSignal(bool);

    void displayPasskeySignal(QString, QString,QString);

    void requestConfirmationSignal(QString, QString, QString);

    void receiveFilesSignal(QString, QString,QString,QString,quint64);

    void sendTransferFilesMesgSignal(QStringList);

    void sendTransferDeviceMesgSignal(QString);

    void showTrayWidgetUISignal();

    void activeConnectionSignal(QString,QString,QString,int,int);

    void defaultAdapterChanged();

    void deviceAddSignal(QMap<QString, QVariant>);

    void transferredChangedSignal(quint64,QString);

    void statusChangedSignal(QMap<QString, QVariant>);

    void pairAgentCanceledSignal(QString);

    void devBatteryChangedSignal();

    void activeUserChangedSignal(QString);

    void connectionErrorMsg(QString,int);

    void adapterAutoConnChanged(bool);

private:
    int getDevBatteryLevel(QString address);

    void connectDBusSignals();

    void addNewPairedDevice(QString address);

    bool registerClient();

    bool unregisterClient();

    /** Init adapters attr */
    void InitAdaptersAttr();

    void getAdapterAttr(QString address);

    void setAdapterAttrInside(QString address, QMap<QString, QVariant> mapAttr);

    void adapterAttrChanged(QString address, QMap<QString, QVariant> mapAttr);

    /** Init devices attr of default adapter */
    void getDevAttr(QString address);

    void setDeviceAttrInside(QString address, QMap<QString, QVariant> mapAttr);

    void deviceAttrChanged(QString address, QMap<QString, QVariant> mapAttr);

    QString userName;
    QMap<QString, QVariant> defaultAdapterAttr;
    QMap<QString, QMap<QString, QVariant>> *devicesList;
    QMap<QString, QMap<QString, QVariant>> *pairedList;
    QMap<QString, QMap<QString, QVariant>> *adapterList;

private slots:
    void adapterAddSignalSLot(QMap<QString, QVariant> adapterAttr);

    void adapterChangedSLot(QString address, QMap<QString, QVariant> adapterAttr);

    void adapterAttrChangedSLot(QString address, QMap<QString, QVariant> adapterAttr);

    void updateClientSLot();

    void deviceAddSLot(QMap<QString, QVariant> deviceAttr);

    void deviceAttrChangedSLot(QString address, QMap<QString, QVariant> deviceAttr);

    void devRemoveSignalSLot(QString address, QMap<QString, QVariant> attr);

    void devBatteryChangedSignalSlot(QString address, int battery);

    void adapterRemoveSignalSLot(QString address);

    void receiveFilesSlot(QMap<QString, QVariant> info);

    void devPairSignalSLot(QMap<QString, QVariant> pairAttr);

    bool getDevSupportFileSend(QString address);

    void devErrorSignalSLot(QString);
};

#endif // BLUETOOTHDBUS_H
