#include "errormessagewidget.h"

ErrorMessageWidget::ErrorMessageWidget(QWidget *parent) : QWidget(parent)
{
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_CLOSE;
    hints.decorations = MWM_DECOR_ALL;
    XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);

    qDebug() << Q_FUNC_INFO << __LINE__;
    this->setWindowIcon(QIcon::fromTheme("bluetooth"));
    this->setWindowTitle(tr("Bluetooth Message"));
    this->setAttribute(Qt::WA_DeleteOnClose);
    this->setBackgroundRole(QPalette::Base);
    this->setAutoFillBackground(true);
#if 0
    if(QGSettings::isSchemaInstalled("org.ukui.style")){
        gsettings = new QGSettings("org.ukui.style");
        connect(gsettings,&QGSettings::changed,this,&ErrorMessageWidget::GSettingsChanges);
    }

    this->setFixedSize(520,160);

    warn_icon = new QLabel(this);
    warn_icon->setFixedSize(22, 22);
    warn_icon->setGeometry(23, 20, 22, 22);
    warn_icon->setPixmap(QIcon::fromTheme("kylin-dialog-warning").pixmap(22,22));
    if (warn_icon->pixmap()->isNull())
        warn_icon->setPixmap(QIcon::fromTheme("dialog-warning").pixmap(22,22));
    warn_icon->setVisible(false);

    warn_title = new QLabel(this);
    warn_title->setVisible(false);

    warn_txt = new QLabel(this);
    warn_txt->setVisible(false);

    ok_btn = new QPushButton(this);
    ok_btn->setVisible(false);
#endif
    QDesktopWidget *deskdop = QApplication::desktop();
    move((deskdop->width()-this->width())/2, (deskdop->height()-this->height())/2);
}

ErrorMessageWidget::~ErrorMessageWidget()
{
    qDebug() << Q_FUNC_INFO;
}

#if 0
void ErrorMessageWidget::GSettingsChanges(QString key) {
    if (key == "systemFontSize" || key == "styleName")
        this->update();
}
#endif

void ErrorMessageWidget::ErrCodeAnalysis(QString dev, int errCode) {
    switch (errCode) {
    case ConnectionErrMsg::ERR_BREDR_CONN_TIMEOUT:
    case ConnectionErrMsg::ERR_BREDR_CONN_PAGE_TIMEOUT:
        TimeOutLayoutInit(dev);
        break;
    default:
        break;
    }
}

void ErrorMessageWidget::TimeOutLayoutInit(QString dev) {
    QString txt = QString(tr("Please confirm if the device '"))
                  + dev
                  + QString(tr("' is in the signal range or the Bluetooth is enabled."));
    showMessageBox(QString(tr("Bluetooth Message")), QString(tr("Connection Time Out!")), txt);
}

void ErrorMessageWidget::showMessageBox(QString title, QString text, QString information) {
    if (msgBoxShowed) {
        mbox->accept();
        mbox->deleteLater();
    }

    emit messageBoxShowed(true);
    msgBoxShowed = true;

    mbox = new QMessageBox(QMessageBox::Warning,title, text, QMessageBox::Ok,this);
    mbox->setInformativeText(information);
    mbox->show();

    int ret = mbox->exec();
    if (ret) {
        msgBoxShowed = false;
        emit messageBoxShowed(false);
    }
}
