﻿#include "trayicon.h"

TrayIcon::TrayIcon(bool _trayshow, bool _existAdapter, QWidget *parent)
    : QWidget(parent)
{
    if(QGSettings::isSchemaInstalled(GSETTING_SCHEMA_UKUIBLUETOOH)){
        settings = new QGSettings(GSETTING_SCHEMA_UKUIBLUETOOH);
        existSettings = true;
    }

    if(QGSettings::isSchemaInstalled(GSETTING_SCHEMA_UKCC)){
        ukccGsetting = new QGSettings(GSETTING_SCHEMA_UKCC,GSETTING_PACH_UKCC);
    }
    existAdapter = _existAdapter;

    bluetooth_tray_icon = new QSystemTrayIcon(this);
    tray_Menu = new QMenu(this);
    QAction *settins_action = new QAction(tr("Set Bluetooth Item"),tray_Menu);
    settins_action->setIcon(QIcon::fromTheme("document-page-setup-symbolic", QIcon(":/res/x/setup.png")) );
    tray_Menu->addAction(settins_action);
    connect(tray_Menu, &QMenu::triggered, this, [=]{
        emit openBluetoothSettings();
    });

    //Create taskbar tray icon and connect to signal slot
    //创建任务栏托盘图标，并连接信号槽
    bluetooth_tray_icon = new QSystemTrayIcon(this);
    bluetooth_tray_icon->setContextMenu(tray_Menu);
    bluetooth_tray_icon->setToolTip(tr("Bluetooth"));

    if (existAdapter && existSettings && _trayshow != settings->get("tray-show").toBool()) {
        _trayshow = settings->get("tray-show").toBool();
        QMap<QString, QVariant> attr;
        attr.remove("trayShow");
        attr.insert("trayShow", QVariant(settings->get("tray-show").toBool()));
        Config::setDefaultAdapterAttr(attr);
    }
    bluetooth_tray_icon->setVisible(_trayshow && existAdapter);

    connect(bluetooth_tray_icon,&QSystemTrayIcon::activated,
            [=](QSystemTrayIcon::ActivationReason reason){
                switch (reason)
                {
                    case QSystemTrayIcon::MiddleClick:
                    case QSystemTrayIcon::DoubleClick: /* 来自于双击激活。 */
                    case QSystemTrayIcon::Trigger: /* 来自于单击激活。 */
                        if (clickEnable)
                            InitTrayWidget();
                        break;
                    case QSystemTrayIcon::Unknown:
                    case QSystemTrayIcon::Context:
                    break;
                }
            });
#ifdef KYSEC
    if (!kysec_is_disabled() && kysec_get_3adm_status() && (getuid() || geteuid())){
        secState = true;
        bluetooth_tray_icon->setVisible(false);
    }
#endif
}

TrayIcon::~TrayIcon()
{
    delete settings;
}

void TrayIcon::InitTrayWidget()
{
    emit showTayWidget();
}

void TrayIcon::SetTrayIcon(bool open)
{
    qDebug() << Q_FUNC_INFO << open;
    if(!open){
        if(QIcon::hasThemeIcon("bluetooth-error"))
            bluetooth_tray_icon->setIcon(QIcon::fromTheme("bluetooth-error"));
        else
            bluetooth_tray_icon->setIcon(QIcon::fromTheme("bluetooth-active-symbolic"));
    }else{
        bluetooth_tray_icon->setIcon(QIcon::fromTheme("bluetooth-active-symbolic"));
    }
}

void TrayIcon::SetAdapterFlag(bool set) {
    qDebug() << Q_FUNC_INFO << __LINE__ ;
    if (secState)
        return;
    if (!existAdapter)
        return;
    bluetooth_tray_icon->setVisible(set);

    if(QGSettings::isSchemaInstalled(GSETTING_SCHEMA_UKUIBLUETOOH)){
        settings->set("tray-show", QVariant(set));
    }

    if (set)
        ukccGsetting->set("show",QVariant(true));
    qDebug() << Q_FUNC_INFO << __LINE__ ;
}

void TrayIcon::setClickEnable(bool enable) {
    clickEnable = !enable;
}

void TrayIcon::setAdapterExist(bool exist) {
    existAdapter = exist;
    bluetooth_tray_icon->setVisible(exist);
}
